@@ -2,7 +2,7 @@
  
    GNU Chess frontend
 
-   Copyright (C) 2001-2020 Free Software Foundation, Inc.
+   Copyright (C) 2001-2021 Free Software Foundation, Inc.
 
    GNU Chess is based on the two research programs
    Cobalt by Chua Kong-Sian and Gazebo by Stuart Cracraft.
@@ -372,7 +372,7 @@ static int UserInputIsAValidMove(void)
 static int GetNextLine( char buf[], char line[] )
 {
   char bufaux[BUF_SIZE]="";
-  int i=0;
+  unsigned int i=0;
   int found=0;
 
   for ( i=0; i<strlen( buf ); ++i ) {
@@ -400,7 +400,7 @@ static int GetNextLine( char buf[], char line[] )
  */
 static int GetNextLineNoRemove( char buf[], char line[] )
 {
-  int i=0;
+  unsigned int i=0;
   int found=0;
 
   for ( i=0; i<strlen( buf ); ++i ) {
@@ -539,7 +539,7 @@ void ForwardEngineOutputToUser( void )
   fd_set set[1];
   struct timeval time_val[1];
   int engineinputready=0;
-  char engineinputaux[BUF_SIZE]="";
+  char engineinputaux[BUF_SIZE+1]="";
 
   /* Poll input from engine in non-blocking mode */
   FD_ZERO(set);
@@ -552,11 +552,18 @@ void ForwardEngineOutputToUser( void )
     printf( "Error reading engine input.\n" );
   } else if ( engineinputready > 0 ) {
     /* There are some data from the engine. Read the data */
-    strncpy( engineinputaux, zerochar, BUF_SIZE );
-    nread = read( pipefd_e2a[0], engineinputaux, BUF_SIZE );
+    strncpy( engineinputaux, zerochar, BUF_SIZE+1 );
+    nread = read( pipefd_e2a[0], engineinputaux, BUF_SIZE+1 );
     /* Write data to output */
-    assert( nread < BUF_SIZE-1 );
-    engineinputaux[nread] = '\0';
-    ssize_t r = write( STDOUT_FILENO, engineinputaux, nread );  // TODO Handle return value
+    assert( nread <= BUF_SIZE+1 );
+    if (nread < BUF_SIZE+1) {
+      engineinputaux[nread] = '\0';
+    } else {
+      engineinputaux[BUF_SIZE] = '\0';
+    }
+    ssize_t r = write( STDOUT_FILENO, engineinputaux, nread );
+    if ( r == -1 ) {
+      printf( "Error sending message to engine.\n" );
+    }
   }
 }
