@@ -1,6 +1,6 @@
 /* GNU Chess 6 - main.cc - entry point
 
-   Copyright (c) 2001-2020 Free Software Foundation, Inc.
+   Copyright (c) 2001-2021 Free Software Foundation, Inc.
 
    GNU Chess is based on the two research programs
    Cobalt by Chua Kong-Sian and Gazebo by Stuart Cracraft.
@@ -218,8 +218,8 @@ extern char userinputbuf[];
 
 void EndProg( int sig __attribute__ ((unused)) )
 {
-    int r = system( "stty sane" );  // TODO Handle return value
-    exit(0);
+    int r = system( "stty sane" );
+    exit(r);
 }
 
 int main (int argc, char *argv[])
@@ -391,7 +391,7 @@ Copyright (C) %s Free Software Foundation, Inc.\n\
 License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n"),
-             "2020" );
+             "2021" );
   }
 
   /* If a usage statement is required output it here */
